'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('swap.app'),
    constants = _require.constants;

var SwapAuth = require('swap.auth');
var SwapRoom = require('swap.room');
var SwapOrders = require('swap.orders');

var _require2 = require('swap.swaps'),
    EthSwap = _require2.EthSwap,
    EthTokenSwap = _require2.EthTokenSwap,
    BtcSwap = _require2.BtcSwap;

var _require3 = require('swap.flows'),
    ETH2BTC = _require3.ETH2BTC,
    BTC2ETH = _require3.BTC2ETH,
    ETHTOKEN2BTC = _require3.ETHTOKEN2BTC,
    BTC2ETHTOKEN = _require3.BTC2ETHTOKEN;

var eth = require('../instances/ethereum');
var btc = require('../instances/bitcoin');

var Ipfs = require('ipfs');
var IpfsRoom = require('ipfs-pubsub-room');

var common = require('./common');

var tokenSwap = require('./tokenSwap');

var setupLocalStorage = require('./setupLocalStorage');

var _require4 = require('node-localstorage'),
    LocalStorage = _require4.LocalStorage;

var sessionStorage = require('node-sessionstorage');

module.exports = function (config) {
  return function (_ref) {
    var account = _ref.account,
        _ref$contracts = _ref.contracts,
        ETH = _ref$contracts.ETH,
        TOKEN = _ref$contracts.TOKEN,
        custom = (0, _objectWithoutProperties3.default)(_ref, ['account', 'contracts']);

    config = (0, _extends3.default)({}, common, config, custom, {

      swapAuth: (0, _extends3.default)({}, common.swapAuth, config.swapAuth, custom.swapAuth),

      swapRoom: (0, _extends3.default)({}, common.swapRoom, config.swapRoom, custom.swapRoom)
    });

    setupLocalStorage();

    var storage = new LocalStorage(config.storageDir);

    var web3 = eth[config.network]().core;
    var bitcoin = btc[config.network]().core;

    var tokens = (config.ERC20TOKENS || []).map(function (_token) {
      return (0, _extends3.default)({ network: config.network }, _token);
    }).filter(function (_token) {
      return _token.network === config.network;
    });

    return {
      network: config.network,
      constants: constants,
      env: (0, _extends3.default)({
        web3: web3,
        bitcoin: bitcoin,
        // bcash,
        Ipfs: Ipfs,
        IpfsRoom: IpfsRoom,
        storage: storage,
        sessionStorage: sessionStorage
      }, config.env),
      services: [new SwapAuth((0, _extends3.default)({
        eth: account,
        btc: null
      }, config.swapAuth)), new SwapRoom(config.swapRoom), new SwapOrders()],

      swaps: [new EthSwap(config.ethSwap(ETH)), new BtcSwap(config.btcSwap()),
      /*config.network === 'mainnet'
        ? new UsdtSwap(config.usdtSwap())
        : null,*/

      new EthTokenSwap(config.noxonTokenSwap(TOKEN)), new EthTokenSwap(config.swapTokenSwap(TOKEN))].concat((0, _toConsumableArray3.default)(config.swaps || []), (0, _toConsumableArray3.default)(tokens.map(function (_token) {
        return new EthTokenSwap(tokenSwap(_token)());
      }))).filter(function (a) {
        return !!a;
      }),

      flows: [ETH2BTC, BTC2ETH, ETHTOKEN2BTC(constants.COINS.noxon), BTC2ETHTOKEN(constants.COINS.noxon), ETHTOKEN2BTC(constants.COINS.swap), BTC2ETHTOKEN(constants.COINS.swap)].concat((0, _toConsumableArray3.default)(config.flows || []), (0, _toConsumableArray3.default)([].concat.apply([], tokens.map(function (_ref2) {
        var name = _ref2.name;
        return [ETHTOKEN2BTC(name), BTC2ETHTOKEN(name)];
      })) || []))
    };
  };
};