'use strict';

var bitcoin = require('../instances/bitcoin').testnet();
var ethereum = require('../instances/ethereum').testnet();
var tokenSwap = require('./tokenSwap');

var id = parseInt(process.argv[2]) || process.env.SERVER_ID || process.env.ACCOUNT || Math.random().toString().slice(2);

var offset = process.env.OFFSET || process.argv[1];
var ROOT_DIR = process.env.ROOT_DIR || '.';

module.exports = {
  id: id,
  network: 'testnet',
  storageDir: ROOT_DIR + '/.storage/__testnet__' + id + '__',
  swapRoom: {
    roomName: 'testnet.swap.online',
    repo: ROOT_DIR + '/.ipfs/__testnet__' + id + '__/' + offset
  },
  ethSwap: function ethSwap() {
    return {
      gasLimit: 2e5,
      address: '0x6F54CDAE7c98b0306fB3aB4daED4cAEe25b92Bc6',
      abi: [{ "constant": false, "inputs": [{ "name": "_secret", "type": "bytes32" }, { "name": "_ownerAddress", "type": "address" }], "name": "withdraw", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function" }, { "constant": true, "inputs": [{ "name": "_participantAddress", "type": "address" }], "name": "getSecret", "outputs": [{ "name": "", "type": "bytes32" }], "payable": false, "stateMutability": "view", "type": "function" }, { "constant": true, "inputs": [{ "name": "", "type": "address" }, { "name": "", "type": "address" }], "name": "participantSigns", "outputs": [{ "name": "", "type": "uint256" }], "payable": false, "stateMutability": "view", "type": "function" }, { "constant": false, "inputs": [{ "name": "_secret", "type": "bytes32" }, { "name": "participantAddress", "type": "address" }], "name": "withdrawNoMoney", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function" }, { "constant": true, "inputs": [], "name": "owner", "outputs": [{ "name": "", "type": "address" }], "payable": false, "stateMutability": "view", "type": "function" }, { "constant": false, "inputs": [{ "name": "_secretHash", "type": "bytes20" }, { "name": "_participantAddress", "type": "address" }, { "name": "_targetWallet", "type": "address" }], "name": "createSwapTarget", "outputs": [], "payable": true, "stateMutability": "payable", "type": "function" }, { "constant": true, "inputs": [{ "name": "", "type": "address" }, { "name": "", "type": "address" }], "name": "swaps", "outputs": [{ "name": "targetWallet", "type": "address" }, { "name": "secret", "type": "bytes32" }, { "name": "secretHash", "type": "bytes20" }, { "name": "createdAt", "type": "uint256" }, { "name": "balance", "type": "uint256" }], "payable": false, "stateMutability": "view", "type": "function" }, { "constant": false, "inputs": [{ "name": "_ownerAddress", "type": "address" }, { "name": "_participantAddress", "type": "address" }], "name": "closeSwapByAdminAfterOneYear", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function" }, { "constant": false, "inputs": [{ "name": "_secretHash", "type": "bytes20" }, { "name": "_participantAddress", "type": "address" }], "name": "createSwap", "outputs": [], "payable": true, "stateMutability": "payable", "type": "function" }, { "constant": false, "inputs": [{ "name": "_secret", "type": "bytes32" }, { "name": "_ownerAddress", "type": "address" }, { "name": "participantAddress", "type": "address" }], "name": "withdrawOther", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function" }, { "constant": true, "inputs": [], "name": "ratingContractAddress", "outputs": [{ "name": "", "type": "address" }], "payable": false, "stateMutability": "view", "type": "function" }, { "constant": true, "inputs": [{ "name": "_ownerAddress", "type": "address" }], "name": "getTargetWallet", "outputs": [{ "name": "", "type": "address" }], "payable": false, "stateMutability": "view", "type": "function" }, { "constant": true, "inputs": [], "name": "admin", "outputs": [{ "name": "", "type": "address" }], "payable": false, "stateMutability": "view", "type": "function" }, { "constant": true, "inputs": [{ "name": "_ownerAddress", "type": "address" }], "name": "getBalance", "outputs": [{ "name": "", "type": "uint256" }], "payable": false, "stateMutability": "view", "type": "function" }, { "constant": false, "inputs": [{ "name": "_participantAddress", "type": "address" }], "name": "refund", "outputs": [], "payable": false, "stateMutability": "nonpayable", "type": "function" }, { "inputs": [], "payable": false, "stateMutability": "nonpayable", "type": "constructor" }, { "anonymous": false, "inputs": [{ "indexed": false, "name": "_buyer", "type": "address" }, { "indexed": false, "name": "_seller", "type": "address" }, { "indexed": false, "name": "_value", "type": "uint256" }, { "indexed": false, "name": "_secretHash", "type": "bytes20" }, { "indexed": false, "name": "createdAt", "type": "uint256" }], "name": "CreateSwap", "type": "event" }, { "anonymous": false, "inputs": [{ "indexed": false, "name": "_buyer", "type": "address" }, { "indexed": false, "name": "_seller", "type": "address" }, { "indexed": false, "name": "_secretHash", "type": "bytes20" }, { "indexed": false, "name": "withdrawnAt", "type": "uint256" }], "name": "Withdraw", "type": "event" }, { "anonymous": false, "inputs": [{ "indexed": false, "name": "_buyer", "type": "address" }, { "indexed": false, "name": "_seller", "type": "address" }], "name": "Close", "type": "event" }, { "anonymous": false, "inputs": [{ "indexed": false, "name": "_buyer", "type": "address" }, { "indexed": false, "name": "_seller", "type": "address" }, { "indexed": false, "name": "_secretHash", "type": "bytes20" }], "name": "Refund", "type": "event" }],
      fetchBalance: function fetchBalance(address) {
        return ethereum.fetchBalance(address);
      },
      estimateGasPrice: function estimateGasPrice() {
        var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
            speed = _ref.speed;

        return ethereum.estimateGasPrice({ speed: speed });
      }
    };
  },
  btcSwap: function btcSwap() {
    return {
      fetchBalance: function fetchBalance(address) {
        return bitcoin.fetchBalance(address);
      },
      fetchUnspents: function fetchUnspents(scriptAddress) {
        return bitcoin.fetchUnspents(scriptAddress);
      },
      broadcastTx: function broadcastTx(txRaw) {
        return bitcoin.broadcastTx(txRaw);
      },
      fetchTxInfo: function fetchTxInfo(txid) {
        return bitcoin.fetchTxInfo(txid);
      },
      estimateFeeValue: function estimateFeeValue() {
        var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
            inSatoshis = _ref2.inSatoshis,
            speed = _ref2.speed,
            address = _ref2.address,
            txSize = _ref2.txSize;

        return bitcoin.estimateFeeValue({ inSatoshis: inSatoshis, speed: speed, address: address, txSize: txSize });
      },
      checkWithdraw: function checkWithdraw(scriptAddress) {
        return bitcoin.checkWithdraw(scriptAddress);
      }
    };
  },
  noxonTokenSwap: tokenSwap({
    network: 'testnet',
    name: 'NOXON',
    decimals: 0,
    tokenAddress: '0x60c205722c6c797c725a996cf9cca11291f90749'
  }),
  swapTokenSwap: tokenSwap({
    network: 'testnet',
    name: 'SWAP',
    decimals: 18,
    tokenAddress: '0xbaa3fa2ed111f3e8488c21861ea7b7dbb5a7b121'
  }),
  cashSwap: function cashSwap() {
    return {
      url: 'http://net2pay.o.atwinta.ru',
      fetchBalance: function fetchBalance() {},
      getURL: function getURL(url) {
        return open(url);
      },
      openURL: function openURL(url) {
        return open(url);
      }
    };
  }
};