'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getAllFinished = exports.removeFinished = exports.saveFinished = exports.getAllInProgress = exports.removeInProgress = exports.saveInProgress = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _swap = require('swap.app');

var _swap2 = _interopRequireDefault(_swap);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var app = void 0;

var checkHistoryInit = function checkHistoryInit(_app) {
  if (!_app) throw new Error('history not initialized: call history.init(app)');
};

var init = function init(_app) {
  _swap2.default.required(_app);

  app = _app;
};

var save = function save(swapID) {
  var storageKey = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'history';

  checkHistoryInit(app);

  var storage = app.env.storage;

  var history = storage.getItem(storageKey) || [];

  try {
    if (history.filter(function (_swapID) {
      return _swapID === swapID;
    }).length > 0) {
      return (0, _debug2.default)('swap.core:simple:history')(storageKey + ': swap already saved id = ' + swapID);
    }

    var newHistory = [].concat((0, _toConsumableArray3.default)(history), [swapID]);

    storage.setItem(storageKey, newHistory);

    (0, _debug2.default)('swap.core:simple:history')(storageKey + ': saved swap = ' + swapID);
  } catch (err) {
    (0, _debug2.default)('swap.core:simple:history')('Error:', storageKey + ': ' + err);
    (0, _debug2.default)('swap.core:simple:history')('Error:', storageKey + ': cannot save swap.history, rewind back');
  }
};

var remove = function remove(swapID) {
  var storageKey = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'history';

  checkHistoryInit(app);

  var storage = app.env.storage;

  var history = storage.getItem(storageKey) || [];

  try {
    if (history.filter(function (_swapID) {
      return _swapID === swapID;
    }).length == 0) {
      return (0, _debug2.default)('swap.core:simple:history')(storageKey + ': swap not saved id = ' + swapID + ' cant remove');
    }

    var newHistory = history.filter(function (_swapID) {
      return _swapID !== swapID;
    });

    storage.setItem(storageKey, newHistory);

    (0, _debug2.default)('swap.core:simple:history')(storageKey + ': remove swap = ' + swapID);
  } catch (err) {
    (0, _debug2.default)('swap.core:simple:history')('Error:', storageKey + ': ' + err);
    (0, _debug2.default)('swap.core:simple:history')('Error:', storageKey + ': cannot save swap.history, rewind back');
  }
};

var getAll = function getAll() {
  var storageKey = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'history';

  checkHistoryInit(app);

  var storage = app.env.storage;

  var history = storage.getItem(storageKey) || [];

  (0, _debug2.default)('swap.core:simple:history')(storageKey + ': history = ' + history);

  return history;
};

var saveInProgress = exports.saveInProgress = function saveInProgress(swapID) {
  return save(swapID, 'history.inProgress');
};
var removeInProgress = exports.removeInProgress = function removeInProgress(swapID) {
  return remove(swapID, 'history.inProgress');
};
var getAllInProgress = exports.getAllInProgress = function getAllInProgress() {
  return getAll('history.inProgress');
};

var saveFinished = exports.saveFinished = function saveFinished(swapID) {
  return save(swapID, 'history.finished');
};
var removeFinished = exports.removeFinished = function removeFinished(swapID) {
  return remove(swapID, 'history.finished');
};
var getAllFinished = exports.getAllFinished = function getAllFinished() {
  return getAll('history.finished');
};

module.exports = {
  init: init,

  save: save,
  remove: remove,
  getAll: getAll,

  saveInProgress: saveInProgress,
  removeInProgress: removeInProgress,
  getAllInProgress: getAllInProgress,

  saveFinished: saveFinished,
  removeFinished: removeFinished,
  getAllFinished: getAllFinished
};