'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var request = require('request-promise-native');

var responseCacheStorage = {};

var responseCacheGetKey = function responseCacheGetKey(req, opts) {
  return opts.method + '-' + opts.endpoint;
};

var responseCacheGet = function responseCacheGet(req, opts) {
  var cacheKey = responseCacheGetKey(req, opts);

  if (opts && opts.cacheResponse && responseCacheStorage[cacheKey] && responseCacheStorage[cacheKey].cacheResponseCreateTime + responseCacheStorage[cacheKey].cacheResponse >= new Date().getTime()) {
    return responseCacheStorage[cacheKey];
  }return false;
};

var responseCacheGetTimeout = function responseCacheGetTimeout(req, opts) {
  var cacheKey = responseCacheGetKey(req, opts);

  if (opts && opts.cacheResponse && responseCacheStorage[cacheKey]) {
    return responseCacheStorage[cacheKey];
  }return false;
};

var responseCacheAdd = function responseCacheAdd(req, opts, resData) {
  var cacheKey = responseCacheGetKey(req, opts);
  var cacheResponse = opts.cacheResponse;
  var cacheResponseCreateTime = new Date().getTime();

  responseCacheStorage[cacheKey] = {
    cacheResponse: cacheResponse,
    cacheResponseCreateTime: cacheResponseCreateTime,
    resData: resData
  };
};

var responseQueryArray = new Array();
var responseQueryWorker = function responseQueryWorker() {
  if (responseQueryArray.length) {
    var currentQuery = responseQueryArray.shift();
    currentQuery(responseQueryWorker);
  } else {
    setTimeout(responseQueryWorker, 10);
  }
};
responseQueryWorker();

var createResponseHandler = function createResponseHandler(req, opts) {
  var debug = opts.method.toUpperCase() + ' ' + opts.endpoint;

  var responseQueryTimeout = 1000;
  // no cache - do request
  return new _promise2.default(function (fulfill, reject) {
    var doRequest = function doRequest(nextTick) {
      // cached answer
      var cachedAnswer = responseCacheGet(req, opts);

      if (cachedAnswer) {
        setTimeout(nextTick, responseQueryTimeout);
        fulfill(cachedAnswer.resData);
        return;
      }
      req.then(function (answer) {
        if (opts.cacheResponse) {
          responseCacheAdd(req, opts, answer);
        }
        setTimeout(nextTick, responseQueryTimeout);
        fulfill(answer);
      }).catch(function (error) {
        setTimeout(nextTick, responseQueryTimeout);
        if (opts.cacheOnFail) {
          var cachedData = responseCacheGetTimeout(req, opts);
          if (cachedData) {
            fulfill(cachedData.resData);
            return;
          }
        }
        reject(error);
      });
    };
    if (opts.queryResponse) {
      responseQueryArray.push(doRequest);
    } else {
      doRequest(function () {});
    }
  });
};

var defaultOptions = {
  sameOrigin: false,
  modifyResult: function modifyResult(resData) {
    return resData;
  },
  onComplete: function onComplete() {}

  /**
   *
   * @param {Object} options
   * @param {String} options.endpoint
   * @param {String} options.method
   * @param {Object} options.headers
   * @param {Object} options.query
   * @param {Object} options.body
   * @param {number} options.delay
   */
};var sendRequest = function sendRequest(options) {
  var opts = (0, _extends3.default)({}, defaultOptions, options);

  var req = void 0;

  if (opts.body) {
    req = request[opts.method](opts.endpoint, { json: !!opts.json, body: opts.body });
  } else {
    req = request[opts.method](opts.endpoint);
  }

  if (opts.timeout) {
    req.timeout((0, _extends3.default)({
      response: 5000, // Wait 5 seconds for the server to start sending,
      deadline: 60000 }, opts.timeout));
  }

  if (opts.query) {
    req.query(opts.query);
  }

  if (opts.sameOrigin) {
    req.withCredentials();
  }

  var responseHandler = createResponseHandler(req, opts);

  responseHandler.abort = req.abort.bind(req);

  return responseHandler;
};

var requestByMethod = function requestByMethod(method) {
  return function (endpoint, opts) {
    return sendRequest((0, _extends3.default)({}, opts, { endpoint: endpoint, method: method }));
  };
};

exports.default = {
  get: requestByMethod('get'),
  post: requestByMethod('post'),
  patch: requestByMethod('patch'),
  put: requestByMethod('put'),
  delete: requestByMethod('delete')
};
module.exports = exports.default;