'use strict';

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//const request = require('request-promise-native')
var request = require('../helpers/request');
var debug = require('debug');

var Web3 = require('web3');

// const MAINNET_PROVIDER = `https://mainnet.infura.io/JCnK5ifEPH9qcQkX0Ahl`
var TESTNET_PROVIDER = 'https://rinkeby.infura.io/v3/5ffc47f65c4042ce847ef66a3fa70d4c';

var MAINNET_PROVIDER = process.env.WEB3_MAINNET_PROVIDER || 'https://geth.swaponline.site';
// const TESTNET_PROVIDER = process.env.WEB3_TESTNET_PROVIDER || `https://tgeth.swaponline.site`
var LOCAL_PROVIDER = process.env.WEB3_LOCAL_PROVIDER || 'http://localhost:7545';

var WEB3_PROVIDERS = {
  mainnet: new Web3.providers.HttpProvider(MAINNET_PROVIDER),
  testnet: new Web3.providers.HttpProvider(TESTNET_PROVIDER)
};

var ETHERCHAIN_API = 'https://www.etherchain.org/api/gasPriceOracle';
var ETHGASSTATION_API = 'https://ethgasstation.info/json/ethgasAPI.json';
var BigNumber = require('bignumber.js');
var TEN = new BigNumber(10);

var ETHERSCAN_APIKEY = '87F9B9IH33JPVRM5ZVFEK1DQTM64FUZFMV';

var filterError = function filterError(error) {
  var name = error.name,
      code = error.code,
      statusCode = error.statusCode,
      options = error.options;


  debug('swap.core:ethereum')('UnknownError: statusCode=' + statusCode + ' ' + error.message);

  throw error;
};

var Ethereum = function () {
  function Ethereum() {
    var _network = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'testnet';

    var _customProvider = arguments[1];
    (0, _classCallCheck3.default)(this, Ethereum);

    var _provider = WEB3_PROVIDERS[_network];

    if (typeof web3 !== 'undefined') {
      this.core = new Web3(web3.currentProvider);
    } else {
      this.core = new Web3(_provider);
    }

    this.etherscan = _network === 'testnet' ? 'https://rinkeby.etherscan.io' : 'https://api.etherscan.io';
  }

  (0, _createClass3.default)(Ethereum, [{
    key: 'fetchBalance',
    value: function fetchBalance(address) {
      var _this = this;

      return this.core.eth.getBalance(address).then(function (wei) {
        var balance = _this.core.utils.fromWei(wei);

        debug('swap.core:ethereum')('ETH Balance:', balance);

        return balance;
      }).catch(function (error) {
        debug('swap.core:ethereum')('ETH error:', error);

        return '0';
      });
    }
  }, {
    key: 'fetchTokenBalance',
    value: function fetchTokenBalance(address, tokenAddress, decimals) {
      var base = TEN.pow(decimals); // 1e18 usually
      var url = this.etherscan + '/api?module=account&action=tokenbalance&contractaddress=' + tokenAddress + '&address=' + address + '&apikey=' + ETHERSCAN_APIKEY;

      // cache 10 seconds
      // query request
      return request.get(url, {
        cacheResponse: 10 * 1000,
        queryResponse: true
      }).then(function (json) {
        return JSON.parse(json);
      }).then(function (_ref) {
        var result = _ref.result;
        return result;
      }).then(function (raw) {
        return BigNumber(raw).dividedBy(base).toString();
      }).catch(function (error) {
        debug('swap.core:ethereum')('TokenBalanceError: ' + error.statusCode + ' ' + url + ' - Failed to fetch token balance (' + tokenAddress + '). Probably too frequent request!');

        return '0';
      });
    }
  }, {
    key: 'sendTransaction',
    value: function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(_ref3) {
        var to = _ref3.to,
            value = _ref3.value;
        var from, gas;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                from = this.core.eth.accounts.wallet[0];
                gas = 1e5;


                value = this.core.utils.toWei(value.toString());

                return _context.abrupt('return', this.core.eth.sendTransaction({ from: from, to: to, value: value, gas: gas }));

              case 4:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function sendTransaction(_x2) {
        return _ref2.apply(this, arguments);
      }

      return sendTransaction;
    }()
  }, {
    key: 'estimateGasPrice',
    value: function () {
      var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(options) {
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.prev = 0;
                _context2.next = 3;
                return this.estimateGasPriceEtherChain(options);

              case 3:
                return _context2.abrupt('return', _context2.sent);

              case 6:
                _context2.prev = 6;
                _context2.t0 = _context2['catch'](0);

                console.error('EstimateFeeError: EtherChain, falling back to EthGasStation estimation...');

              case 9:
                _context2.prev = 9;
                _context2.next = 12;
                return this.estimateGasPriceEthGasStation(options);

              case 12:
                return _context2.abrupt('return', _context2.sent);

              case 15:
                _context2.prev = 15;
                _context2.t1 = _context2['catch'](9);

                console.error('EstimateFeeError: EthGasStation, falling back to Web3 estimation...');

              case 18:
                _context2.next = 20;
                return this.estimateGasPriceWeb3(options);

              case 20:
                return _context2.abrupt('return', _context2.sent);

              case 21:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[0, 6], [9, 15]]);
      }));

      function estimateGasPrice(_x3) {
        return _ref4.apply(this, arguments);
      }

      return estimateGasPrice;
    }()
  }, {
    key: 'estimateGasPriceWeb3',
    value: function () {
      var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        var _this2 = this;

        var _ref6 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
            _ref6$speed = _ref6.speed,
            speed = _ref6$speed === undefined ? 'fast' : _ref6$speed;

        var _multiplier, gasPrice;

        return _regenerator2.default.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _multiplier = function () {
                  switch (speed) {
                    case 'fast':
                      return 2;
                    case 'normal':
                      return 1;
                    case 'slow':
                      return 0.5;
                    default:
                      return 1;
                  }
                }();

                _context3.next = 3;
                return new _promise2.default(function (resolve, reject) {
                  return _this2.core.eth.getGasPrice(function (err, gasPrice) {
                    if (err) {
                      reject(err);
                    } else {
                      resolve(gasPrice);
                    }
                  });
                });

              case 3:
                gasPrice = _context3.sent;
                return _context3.abrupt('return', BigNumber(gasPrice).multipliedBy(_multiplier));

              case 5:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function estimateGasPriceWeb3() {
        return _ref5.apply(this, arguments);
      }

      return estimateGasPriceWeb3;
    }()
  }, {
    key: 'estimateGasPriceEtherChain',
    value: function estimateGasPriceEtherChain() {
      var _ref7 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          _ref7$speed = _ref7.speed,
          speed = _ref7$speed === undefined ? 'fast' : _ref7$speed;

      var _speed = function () {
        switch (speed) {
          case 'fast':
            return 'fast';
          case 'normal':
            return 'standard';
          case 'slow':
            return 'safeLow';
          default:
            return 'standard';
        }
      }();

      // cache 10 minute
      // query request
      // use cache if fail
      return request.get('' + ETHERCHAIN_API, {
        cacheResponse: 10 * 60 * 1000,
        queryResponse: true,
        cacheOnFail: true
      }).then(function (json) {
        return JSON.parse(json);
      }).then(function (fees) {
        return BigNumber(fees[_speed]).multipliedBy(1e9);
      }).catch(function (error) {
        return filterError(error);
      });
    }
  }, {
    key: 'estimateGasPriceEthGasStation',
    value: function estimateGasPriceEthGasStation(_ref8) {
      var _ref8$speed = _ref8.speed,
          speed = _ref8$speed === undefined ? 'fast' : _ref8$speed;

      var _speed = function () {
        switch (speed) {
          case 'fast':
            return 'fast';
          case 'normal':
            return 'average';
          case 'slow':
            return 'safeLow';
          default:
            return 'average';
        }
      }();

      // cache 10 minute
      // query request
      // use cache if fail
      return request.get('' + ETHGASSTATION_API, {
        cacheResponse: 10 * 60 * 1000,
        queryResponse: true,
        cacheOnFail: true
      }).then(function (json) {
        return JSON.parse(json);
      }).then(function (fees) {
        return BigNumber(fees[_speed]).dividedBy(10).multipliedBy(1e9);
      }).catch(function (error) {
        return filterError(error);
      });
    }
  }]);
  return Ethereum;
}();

module.exports = new Ethereum();
module.exports.mainnet = function () {
  return new Ethereum('mainnet');
};
module.exports.testnet = function () {
  return new Ethereum('testnet');
};